<?php

/**
 * @file
 *   drush integration for smsc.
 */

/**
 * The SMSC library URI.
 */
define('SMSC_DOWNLOAD_URI', 'https://github.com/awd-studio/smsc/archive/master.zip');
define('SMSC_DOWNLOAD_PREFIX', 'smsc-');

/**
 * Implementation of hook_drush_command().
 *
 * In this hook, you specify which commands your
 * drush module makes available, what it does and
 * description.
 *
 * Notice how this structure closely resembles how
 * you define menu hooks.
 *
 * See `drush topic docs-commands` for a list of recognized keys.
 *
 * @return array
 *   An associative array describing your command(s).
 */
function smsc_drush_command() {
  $items = [];

  // the key in the $items array is the name of the command.
  $items['smsc-download'] = [
    'callback'    => 'drush_smsc_download',
    'description' => dt('Download and install the SMSC library.'),
    'bootstrap'   => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap.
    'arguments'   => [
      'path' => dt('Optional. A path where to install the SMSC library. If omitted Drush will use the default location.'),
    ],
    'aliases'     => ['smscdownload', 'smsc-dl', 'smscdl'],
  ];

  return $items;
}

/**
 * Implementation of hook_drush_help().
 *
 * This function is called whenever a drush user calls
 * 'drush help <name-of-your-command>'
 *
 * @param string $section
 *   A string with the help section (prepend with 'drush:')
 *
 * @return string
 *   A string with the help text for your command.
 */
function smsc_drush_help($section) {
  switch ($section) {
    case 'drush:smsc-download':
      return dt('Download and install the SMSC library from awd-studio/smsc, default location is sites/all/libraries.');
  }
}

/**
 * Implements drush_MODULE_pre_pm_enable().
 */
function drush_smsc_pre_pm_enable() {
  $modules = drush_get_context('PM_ENABLE_MODULES');
  if (in_array('smsc', $modules) && !drush_get_option('skip')) {
    drush_smsc_download();
  }
}

/**
 * Command to download the SMSC library.
 */
function drush_smsc_download() {
  $args = func_get_args();
  if (!empty($args[0])) {
    $path = $args[0];
  }
  else {
    $path = 'sites/all/libraries';
  }

  // Create the path if it does not exist.
  if (!is_dir($path)) {
    drush_op('mkdir', $path);
    drush_log(dt('Directory @path was created', ['@path' => $path]), 'notice');
  }

  // Download smsc download only if path is writable.
  if (is_writable($path)) {
    // Set the directory to the download location.
    $olddir = getcwd();
    chdir($path);

    // Download the zip archive
    if ($filepath = drush_download_file(SMSC_DOWNLOAD_URI)) {
      $filename = basename($filepath);
      $dirname  = SMSC_DOWNLOAD_PREFIX . basename($filepath, '.zip');

      // Remove any existing SMSC library directory.
      if (is_dir($dirname) || is_dir('smsc')) {
        drush_delete_dir($dirname, TRUE);
        drush_delete_dir('smsc', TRUE);
        drush_log(dt('A existing SMSC library was deleted from @path', ['@path' => $path]), 'notice');
      }

      // Decompress the zip archive
      drush_tarball_extract($filename);

      // Change the directory name to "smsc" if needed.
      if ($dirname != 'smsc') {
        drush_move_dir($dirname, 'smsc', TRUE);
        $dirname = 'smsc';
      }
    }

    if (is_dir($dirname)) {
      drush_log(dt('SMSC library has been installed in @path', ['@path' => $path]), 'success');
    }
    else {
      drush_log(dt('Drush was unable to install the SMSC library to @path', ['@path' => $path]), 'error');
    }

    // Set working directory back to the previous working directory.
    chdir($olddir);
  }
  else {
    drush_log(dt('Drush was unable to install the SMSC library because @path is not writable. If you enable the smsc module before you install the download library, you may find that smsc does not work until you reinstall the smsc module.', ['@path' => $path]), 'warning');
  }
}
